/*
 * A resource offering support for creating contexts and opening files.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/file_object_registry.h>
#include <fsserver/file_opening.h>
#include <fsserver/opener_context_resource.h>
#include <fsserver/opener_interface.h>
#include <fsserver/resource.h>



/* Support for providing access to files. */

class OpenerResource : public Resource, public FileOpening, public Opener
{
protected:
    FileObjectRegistry *_registry;

    /* Convenience methods determining different object types. */

    virtual bool accessing_directory(const char *path, flags_t flags, fileid_t fileid) = 0;

    virtual bool accessing_file(const char *path, flags_t flags, fileid_t fileid) = 0;

    /* Convenience methods obtaining different resource types. */

    virtual long get_directory(const char *path, flags_t flags, fileid_t fileid,
                               offset_t *size, l4_cap_idx_t *cap,
                               object_flags_t *object_flags);

    virtual long get_file(const char *path, flags_t flags, fileid_t fileid,
                          offset_t *size, l4_cap_idx_t *cap,
                          object_flags_t *object_flags);

public:
    explicit OpenerResource(FileObjectRegistry *registry);

    virtual ~OpenerResource();

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<Opener *>(this); }

    /* Direct access methods. */

    long open(const char *path, flags_t flags, offset_t *size,
              l4_cap_idx_t *cap, object_flags_t *object_flags);

    /* Opener interface methods. */

    long context(l4_cap_idx_t *context);
};

// vim: tabstop=4 expandtab shiftwidth=4
